@@ -1,7 +1,7 @@
 context("summary")
 
 test_all <- (identical (Sys.getenv ("MPADGE_LOCAL"), "true") |
-             identical (Sys.getenv ("TRAVIS"), "true"))
+             identical (Sys.getenv ("GITHUB_WORKFLOW"), "test-coverage"))
 
 test_that ("extract non gtfs", {
               f <- file.path (tempdir (), "junk.txt")
