@@ -1,8 +1,7 @@
 context("time formats")
 
 test_all <- (identical (Sys.getenv ("MPADGE_LOCAL"), "true") |
-             identical (Sys.getenv ("TRAVIS"), "true"))
-
+             identical (Sys.getenv ("GITHUB_WORKFLOW"), "test-coverage"))
 
 test_that("convert-time", {
               berlin_gtfs_to_zip ()
@@ -10,7 +9,9 @@ test_that("convert-time", {
               expect_true (file.exists (f))
               expect_silent (g <- extract_gtfs (f, quiet = TRUE))
               expect_silent (gt <- gtfs_timetable (g, quiet = TRUE))
-              day <- strftime (Sys.time (), "%A")
+              days <- c ("monday", "tuesday", "wednesday", "thursday",
+                         "friday", "saturday", "sunday")
+              day <- days [as.integer (strftime (Sys.time (), "%u"))]
               msg <- paste0 ("Day not specified; extracting timetable for ",
                              day)
               expect_message (gt <- gtfs_timetable (g, quiet = FALSE), msg)
@@ -81,3 +82,35 @@ test_that ("day param", {
               expect_error (gt <- gtfs_timetable (g, day = NA),
                             "day must be a day of the week")
              })
+
+test_that ("date param", {
+  berlin_gtfs_to_zip ()
+  f <- file.path (tempdir (), "vbb.zip")
+  expect_true (file.exists (f))
+  expect_silent (g <- extract_gtfs (f, quiet = TRUE))
+
+  # date not in feed
+  expect_error (gt <- gtfs_timetable (g, date = 20180128),
+                "Calendar contains no matching dates")
+
+  # wrong date format
+  expect_error (gt <- gtfs_timetable (g, date = 1234),
+                "Date must be provided in the format YYYYMMDD")
+  expect_error (gt <- gtfs_timetable (g, date = "abc"),
+                "Date must be provided in the format YYYYMMDD")
+
+  gt_day <- gtfs_timetable (g, day = "Monday", quiet = TRUE)
+  g$calendar_dates <- data.table::data.table (service_id = "1",
+                                              date = 20190128,
+                                              exception_type = 1)
+  g$calendar <- g$calendar[service_id != 1]
+  expect_silent (gt <- gtfs_timetable (g, date = 20190128, quiet = TRUE))
+  expect_identical(gt_day$timetable, gt$timetable)
+
+  # sunday added to calendar_dates
+  g$calendar_dates <- data.table::data.table (service_id = "1",
+                                              date = 2019017,
+                                              exception_type = 1)
+  expect_silent (gt <- gtfs_timetable (g, date = 20190127, quiet = TRUE))
+  expect_false(identical(gt_day$timetable, gt$timetable))
+})
